from __future__ import annotations
from typing import Literal, TYPE_CHECKING

import datetime

from .traders import Trader

if TYPE_CHECKING:
    from .http import HTTPClient

    from .types.item import (
        Item as ItemPayload,
        BSGItem as BSGItemPayload,
        BSGPrefab as PrefabPayload
    )
    from .types.trader import Trader as TraderPayload

__all__ = (
    'Item',
    'BSGItem'
)


class Item:

    __slots__ = (
        'uid',
        'bsg_id',
        'name',
        'short_name',
        'price',
        'base_price',
        'slots',
        'avg24h_price',
        'avg7days_price',
        '_updated_at',
        'diff24h',
        'diff7days',
        'link',
        'wiki_link',
        'icon_url',
        'image_url',
        '_http',
        '_trader_payload'
    )

    def __init__(self, http: HTTPClient, payload: ItemPayload):
        self._http = http

        self.uid: str = payload['uid']
        self.bsg_id: str = payload['bsgId']
        self.name: str = payload['name']
        self.short_name: str = payload['shortName']
        self.slots: int = payload['slots']
        self.link: str = payload['link']
        self.wiki_link: str = payload['wikiLink']

        self._update(payload)

    def __str__(self) -> str:
        return self.name

    def __int__(self) -> int:
        return self.price

    def __eq__(self, other) -> bool:
        return isinstance(other, Item) and self.uid == other.uid

    def __ne__(self, other) -> bool:
        return not self.__eq__(other)

    def __repr__(self) -> str:
        attrs = (
            ('uid', self.uid),
            ('name', self.name),
            ('price', self.price),
            ('short_name', self.short_name)
        )

        joined = ' '.join('%s=%r' % t for t in attrs)
        return f'<{self.__class__.__name__} {joined}>'

    def __hash__(self) -> int:
        return hash(self.uid)

    @property
    def trader(self) -> Trader:
        return Trader(self._trader_payload)

    @property
    def url(self):
        return self.link

    @property
    def wiki_url(self):
        return self.wiki_link

    @property
    def updated_at(self) -> datetime.datetime:
        return datetime.datetime.strptime(self._updated_at, "%Y-%m-%dT%H:%M:%S.%fZ")

    def _update(self, data: ItemPayload):
        self.price = data['price']
        self.base_price = data['basePrice']
        self.avg24h_price = data['avg24hPrice']
        self.avg7days_price = data['avg7daysPrice']
        self._updated_at = data['updated']
        self.diff24h = data['diff24h']
        self.diff7days = data['diff7days']

        self.icon_url = data['icon']
        self.image_url = data['imgBig']

        self._trader_payload: TraderPayload = {
            'name': data['traderName'],
            'price': data['traderPrice'],
            'currency': data['traderPriceCur']
        }

    async def update(self) -> None:
        """|coro|

        Update Item data.
        """

        http = self._http

        async with http:
            data = await http.get_item_by_uid(self.uid)

        self._update(data)


class Prefab:

    def __init__(self, prefab_type: int, payload: PrefabPayload):
        self._type: int = prefab_type
        self._update(payload)

    def _update(self, data: PrefabPayload):
        self.path = data['path']
        self.rcid = data['rcid']

    @property
    def type(self):

        types = {
            0: 'basic',
            1: 'user'
        }

        return types.get(self._type, 'Unknown Type')


class BSGItem:

    def __init__(self, payload: BSGItemPayload):
        self.raw_name: str = payload['_name']
        self.id: str = payload['_id']
        self.parent: str = payload['_parent']
        self.type: str = payload['_type']

        self.proto: str = payload.get('_proto', '')

        self._update(payload)

    def _update(self, data: BSGItemPayload) -> None:
        _props = data['_props']

        self._props = _props

        self.name = _props['Name']
        self.short_name = _props['ShortName']
        self.description = _props['Description']

        self.weight: int = _props['Weight']
        self.width: int = _props['Width']
        self.height: int = _props['Height']

        self.stack_max: int = _props['StackMaxSize']
        # self.rarity = _props['Rarity']

        self.spawn_chance: int = _props['SpawnChance']
        self.item_sound = _props['ItemSound']

        self.prefab = Prefab(0, _props['Prefab'])
        self.user_prefab = Prefab(1, _props['UsePrefab'])

        self.examine_time: int = _props['ExamineTime']

        self.loot_experience: int = _props['LootExperience']

        self.is_quest_item: bool = _props['QuestItem']

        self.fold_able: bool = _props['IsAnimated']

    @property
    def loot_exp(self) -> int:
        return self.loot_experience



from __future__ import annotations
from typing import Optional, TYPE_CHECKING

if TYPE_CHECKING:
    from .types.trader import currencies, Trader as TraderPayload


class Trader:

    def __init__(self, payload: TraderPayload):
        self._update(payload)

    def __str__(self) -> str:
        return self.name

    def _update(self, data: TraderPayload):
        name: str = data['name']

        self.name: Optional[str] = None if name == '' else name
        self.price: int = data['price']
        self.currency: currencies = data['currency']



"""
Tarkov Market API Wrapper
~~~~~~~~~~~~~~~~

:copyright: (c) 2021-present Hostagen
:license: MIT, see LICENSE for more details.

"""

__title__ = 'tarkov_market'
__author__ = 'Hostagen'
__license__ = 'MIT'
__version__ = '0.6.1'

from .client import *
from .errors import *
from .item import *
from .traders import *
from . import utils



from typing import TypedDict, Literal

currencies = Literal['₽', '$', '€']


class _RawTrader(TypedDict, total=False):
    traderName: str
    traderPrice: int
    traderPriceCur: currencies


class Trader(TypedDict, total=False):
    name: str
    price: int
    currency: currencies



from setuptools import setup
import re

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

version = ''
with open('tarkov_market/__init__.py') as f:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', f.read(), re.MULTILINE).group(1)

readme = ''

with open('README.rst') as f:
    readme = f.read()

packages = [
    'tarkov_market',
    'tarkov_market.types'
]

setup(
    name='tarkov-market.py',
    url='https://github.com/Hostagen/tarkov-market.py',
    description='async API wrapper for Tarkov Market written in Python.',
    long_description=readme,
    long_description_content_type="text/x-rst",
    version=version,
    packages=packages,
    author='Hostagen',
    license='MIT',
    python_requires='>=3.8.0',
)



from typing import List, Dict, Literal, TypedDict
from .trader import _RawTrader

LangType = Literal['en', 'ru', 'de', 'fr', 'es', 'cn']


class Item(_RawTrader):
    uid: str
    bsgId: str
    name: str
    shortName: str
    price: int
    basePrice: int
    avg24hPrice: int
    avg7daysPrice: int
    updated: str
    slots: int
    diff24h: int
    diff7days: int
    icon: str
    link: str
    wikiLink: str
    img: str
    imgBig: str
    reference: str


class Point3D(TypedDict, total=False):
    x: int
    y: int
    z: int


class BSGPrefab(TypedDict, total=False):
    path: str
    rcid: str


class Filter(TypedDict, total=False):
    Shift: int
    Filter: List[str]


class Filters(TypedDict, total=False):
    filters: Filter


class _BaseItem(TypedDict):
    _id: str
    _name: str
    _parent: str
    _proto: str


class StackSlots(_BaseItem, total=False):
    _props: Filters
    _max_count: int


ThrowType = Literal['frag_grenade', 'smoke_grenade', 'flash_grenade']
FragmentType = Literal['5996f6d686f77467977ba6cc', '5996f6fc86f7745e585b4de3']
ammoType = Literal['bullet', 'grenade', 'buckshot']


class BSGProps(TypedDict, total=False):

    # Basic Data
    Name: str
    ShortName: str
    Description: str
    Weight: int
    BackgroundColor: str
    Width: int
    Height: int
    StackMaxSize: int
    Rarity: str
    SpawnChance: int
    CreditsPrice: int
    ItemSound: str
    Prefab: BSGPrefab
    UsePrefab: BSGPrefab
    StackObjectsCount: int
    NotShownInSlot: bool
    ExaminedByDefault: bool
    ExamineTime: int

    IsUndiscardable: bool
    IsUnsaleable: bool
    IsUnbuyable: bool
    IsUngivable: bool
    IsLockedafterEquip: bool

    QuestItem: bool
    LootExperience: int
    ExamineExperience: int
    HideEntrails: bool
    RepairCost: int
    RepairSpeed: int
    ExtraSizeLeft: int
    ExtraSizeRight: int
    ExtraSizeUp: int
    ExtraSizeDown: int
    ExtraSizeForceAdd: bool
    MergesWithChildren: bool
    CanSellOnRagfair: bool
    CanRequireOnRagfair: bool
    ConflictingItems: list
    FixedPrice: bool
    Unlootable: bool
    UnlootableFromSlot: str
    UnlootableFromSide: list
    ChangePriceCoef: int
    AllowSpawnOnLocations: list
    SendToClient: bool
    AnimationVariantsNumber: int
    DiscardingBlock: bool
    RagFairCommissionModifier: int
    IsAlwaysAvailableForInsurance: bool

    StackSlots: StackSlots
    IsAnimated: bool

    # Ammo Data
    ammoType: ammoType
    ammoCaliber: str
    Damage: int
    ammoAccr: int
    ammoRec: int
    ammoDist: int
    buckshotBullets: int

    PenetrationPower: int
    PenetrationPowerDiviation: int

    ammoHear: int
    ammoSfx: str

    MisfireChance: int

    # Grenade Data
    ThrowType: ThrowType
    throwDamMax: int
    Strength: int
    EmitTime: int

    MinExplosionDistance: int
    MaxExplosionDistance: int

    ExplDelay: int
    explDelay: int

    throwDamMax: int

    CanBeHiddenDuringThrow: bool

    # Fragment Data
    FragmentsCount: int
    FragmentType: FragmentType

    ContusionDistance: int

    Blindness: Point3D
    Contusion: Point3D
    ArmorDistanceDistanceDamage: Point3D

    # Weapon Data
    Foldable: bool


class BSGItem(_BaseItem, total=False):
    _type: str
    _props: BSGProps
